/*
 * Copyright [2022] [https://www.xiaonuo.vip]
 *
 * Snowy采用APACHE LICENSE 2.0开源协议，您在使用过程中，需要注意以下几点：
 *
 * 1.请不要删除和修改根目录下的LICENSE文件。
 * 2.请不要删除和修改Snowy源码头部的版权声明。
 * 3.本项目代码可免费商业使用，商业使用请保留源码和相关描述文件的项目出处，作者声明等。
 * 4.分发源码时候，请注明软件出处 https://www.xiaonuo.vip
 * 5.不可二次分发开源参与同类竞品，如有想法可联系团队xiaonuobase@qq.com商议合作。
 * 6.若您的项目无法满足以上几点，需要更多功能代码，获取Snowy商业授权许可，请在官网购买授权，地址为 https://www.xiaonuo.vip
 */
package vip.xiaonuo.biz.modular.bizinventory.controller;

import cn.dev33.satoken.annotation.SaCheckPermission;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.github.xiaoymin.knife4j.annotations.ApiOperationSupport;
import com.github.xiaoymin.knife4j.annotations.ApiSupport;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import org.springframework.http.MediaType;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RestController;
import vip.xiaonuo.biz.modular.bizinventory.param.*;
import vip.xiaonuo.common.annotation.CommonLog;
import vip.xiaonuo.common.pojo.CommonResult;
import vip.xiaonuo.common.pojo.CommonValidList;
import vip.xiaonuo.biz.modular.bizinventory.entity.BizInventory;
import vip.xiaonuo.biz.modular.bizinventory.service.BizInventoryService;

import javax.annotation.Resource;
import javax.servlet.http.HttpServletResponse;
import javax.validation.Valid;
import javax.validation.constraints.NotEmpty;
import java.io.IOException;
import java.util.List;
import java.util.Map;

/**
 * 出入库管理控制器
 *
 * @author fanzherong
 * @date  2025/02/20 10:02
 */
@Api(tags = "出入库管理控制器")
@ApiSupport(author = "SNOWY_TEAM", order = 1)
@RestController
@Validated
public class BizInventoryController {

    @Resource
    private BizInventoryService bizInventoryService;

    /**
     * 获取出入库管理分页
     *
     * @author fanzherong
     * @date  2025/02/20 10:02
     */
    @ApiOperationSupport(order = 1)
    @ApiOperation("获取出入库管理分页")
    @SaCheckPermission("/biz/bizinventory/page")
    @GetMapping("/biz/bizinventory/page")
    public CommonResult<Page<BizInventory>> page(BizInventoryPageParam bizInventoryPageParam) {
        return CommonResult.data(bizInventoryService.page(bizInventoryPageParam));
    }

    /**
     * 添加出入库管理
     *
     * @author fanzherong
     * @date  2025/02/20 10:02
     */
    @ApiOperationSupport(order = 2)
    @ApiOperation("添加出入库管理")
    @CommonLog("添加出入库管理")
    @SaCheckPermission("/biz/bizinventory/add")
    @PostMapping("/biz/bizinventory/add")
    public CommonResult<String> add(@RequestBody @Valid BizInventoryAddParam bizInventoryAddParam) {
        bizInventoryService.add(bizInventoryAddParam);
        return CommonResult.ok();
    }

    /**
     * 编辑出入库管理
     *
     * @author fanzherong
     * @date  2025/02/20 10:02
     */
    @ApiOperationSupport(order = 3)
    @ApiOperation("编辑出入库管理")
    @CommonLog("编辑出入库管理")
    @SaCheckPermission("/biz/bizinventory/edit")
    @PostMapping("/biz/bizinventory/edit")
    public CommonResult<String> edit(@RequestBody @Valid BizInventoryEditParam bizInventoryEditParam) {
        bizInventoryService.edit(bizInventoryEditParam);
        return CommonResult.ok();
    }

    /**
     * 删除出入库管理
     *
     * @author fanzherong
     * @date  2025/02/20 10:02
     */
    @ApiOperationSupport(order = 4)
    @ApiOperation("删除出入库管理")
    @CommonLog("删除出入库管理")
    @SaCheckPermission("/biz/bizinventory/delete")
    @PostMapping("/biz/bizinventory/delete")
    public CommonResult<String> delete(@RequestBody @Valid @NotEmpty(message = "集合不能为空")
                                                   CommonValidList<BizInventoryIdParam> bizInventoryIdParamList) {
        bizInventoryService.delete(bizInventoryIdParamList);
        return CommonResult.ok();
    }

    /**
     * 获取出入库管理详情
     *
     * @author fanzherong
     * @date  2025/02/20 10:02
     */
    @ApiOperationSupport(order = 5)
    @ApiOperation("获取出入库管理详情")
    @SaCheckPermission("/biz/bizinventory/detail")
    @GetMapping("/biz/bizinventory/detail")
    public CommonResult<BizInventory> detail(@Valid BizInventoryIdParam bizInventoryIdParam) {
        return CommonResult.data(bizInventoryService.detail(bizInventoryIdParam));
    }

    /**
     * 出入库统计
     * @param bizInventoryPageParam
     * @return
     */
    @ApiOperationSupport(order = 5)
    @ApiOperation("出入库统计")
    @SaCheckPermission("/biz/bizinventory/getTotal")
    @GetMapping("/biz/bizinventory/getTotal")
    public CommonResult<Map<String,Object>> getTotal(BizInventoryPageParam bizInventoryPageParam){
        return CommonResult.data(bizInventoryService.getTotal(bizInventoryPageParam));
    }

    /**
     * 导出出入库报表
     */
    @SaCheckPermission("/biz/bizinventory/exportRecord")
    @GetMapping(value = "/biz/bizinventory/exportRecord", produces = MediaType.APPLICATION_OCTET_STREAM_VALUE)
    public void exportRecord(BizInventoryPageParam bizInventoryPageParam, HttpServletResponse response) throws IOException {
        bizInventoryService.exportRecord(bizInventoryPageParam,response);
    }

    /**
     * 获取统计
     *
     * @author xiaofan
     * @date  2023/10/11 16:33
     */
    @ApiOperationSupport(order = 1)
    @ApiOperation("获取统计")
    @SaCheckPermission("/biz/bizinventory/getReportList")
    @GetMapping("/biz/bizinventory/getReportList")
    public CommonResult<List<BizInventoryReport>> getReportList(BizInventoryParam bizInventoryParam) {
        return CommonResult.data(bizInventoryService.getReportList(bizInventoryParam));
    }

    /**
     * 导出出入库报表
     */
    @SaCheckPermission("/biz/bizinventory/export")
    @GetMapping(value = "/biz/bizinventory/export", produces = MediaType.APPLICATION_OCTET_STREAM_VALUE)
    public void exportRecord(BizInventoryParam bizInventoryParam , HttpServletResponse response) throws IOException {
        bizInventoryService.exportReport(bizInventoryParam,response);
    }
}
