/*
 * Copyright [2022] [https://www.xiaonuo.vip]
 *
 * Snowy采用APACHE LICENSE 2.0开源协议，您在使用过程中，需要注意以下几点：
 *
 * 1.请不要删除和修改根目录下的LICENSE文件。
 * 2.请不要删除和修改Snowy源码头部的版权声明。
 * 3.本项目代码可免费商业使用，商业使用请保留源码和相关描述文件的项目出处，作者声明等。
 * 4.分发源码时候，请注明软件出处 https://www.xiaonuo.vip
 * 5.不可二次分发开源参与同类竞品，如有想法可联系团队xiaonuobase@qq.com商议合作。
 * 6.若您的项目无法满足以上几点，需要更多功能代码，获取Snowy商业授权许可，请在官网购买授权，地址为 https://www.xiaonuo.vip
 */
package vip.xiaonuo.biz.modular.bizinventory.param;

import io.swagger.annotations.ApiModelProperty;
import lombok.Getter;
import lombok.Setter;

import javax.validation.constraints.NotBlank;
import javax.validation.constraints.NotNull;
import java.math.BigDecimal;
import java.util.Date;
import java.util.List;

/**
 * 出入库管理编辑参数
 *
 * @author fanzherong
 * @date  2025/02/20 10:02
 **/
@Getter
@Setter
public class BizInventoryEditParam {

    /** 主键ID */
    @ApiModelProperty(value = "主键ID", required = true, position = 1)
    @NotBlank(message = "id不能为空")
    private String id;

    /** 车牌号码 */
    @ApiModelProperty(value = "车牌号码", required = true, position = 2)
    @NotBlank(message = "licenseNumber不能为空")
    private String licenseNumber;

    /** 品名 */
    @ApiModelProperty(value = "品名", required = true, position = 3)
    @NotBlank(message = "productName不能为空")
    private String productName;

    /** 司机姓名 */
    @ApiModelProperty(value = "司机姓名", required = true, position = 4)
    @NotBlank(message = "driverName不能为空")
    private String driverName;

    /** 手机号 */
    @ApiModelProperty(value = "手机号", required = true, position = 5)
    @NotBlank(message = "driverPhone不能为空")
    private String driverPhone;

    /** 证件号 */
    @ApiModelProperty(value = "证件号", required = true, position = 6)
    @NotBlank(message = "idNumber不能为空")
    private String idNumber;

    /** 车辆毛重 */
    @ApiModelProperty(value = "车辆毛重", required = true, position = 7)
    @NotNull(message = "grossWeight不能为空")
    private BigDecimal grossWeight;

    /** 车辆皮重 */
    @ApiModelProperty(value = "车辆皮重", required = true, position = 8)
    @NotNull(message = "tareWeight不能为空")
    private BigDecimal tareWeight;

    /** 出入库类型 */
    @ApiModelProperty(value = "出入库类型", required = true, position = 9)
    @NotBlank(message = "inventoryType不能为空")
    private String inventoryType;

    /** 包数量 */
    @ApiModelProperty(value = "包数量", required = true, position = 10)
    @NotNull(message = "inventoryWeight不能为空")
    private BigDecimal inventoryWeight;

    /** 出入库时间 */
    @ApiModelProperty(value = "出入库时间", required = true, position = 11)
    @NotNull(message = "inventoryTime不能为空")
    private Date inventoryTime;

    /** 附件名称 */
    @ApiModelProperty(value = "附件名称", position = 12)
    private String fileName;

    /** 附件地址 */
    @ApiModelProperty(value = "附件地址", position = 13)
    private String filePath;

    /**文件id集合*/
    private List<String> fileIdList;

}
