/*
 * Copyright [2022] [https://www.xiaonuo.vip]
 *
 * Snowy采用APACHE LICENSE 2.0开源协议，您在使用过程中，需要注意以下几点：
 *
 * 1.请不要删除和修改根目录下的LICENSE文件。
 * 2.请不要删除和修改Snowy源码头部的版权声明。
 * 3.本项目代码可免费商业使用，商业使用请保留源码和相关描述文件的项目出处，作者声明等。
 * 4.分发源码时候，请注明软件出处 https://www.xiaonuo.vip
 * 5.不可二次分发开源参与同类竞品，如有想法可联系团队xiaonuobase@qq.com商议合作。
 * 6.若您的项目无法满足以上几点，需要更多功能代码，获取Snowy商业授权许可，请在官网购买授权，地址为 https://www.xiaonuo.vip
 */
package vip.xiaonuo.biz.modular.bizuservideo.controller;

import cn.dev33.satoken.annotation.SaCheckPermission;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.github.xiaoymin.knife4j.annotations.ApiOperationSupport;
import com.github.xiaoymin.knife4j.annotations.ApiSupport;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RestController;
import vip.xiaonuo.common.annotation.CommonLog;
import vip.xiaonuo.common.pojo.CommonResult;
import vip.xiaonuo.common.pojo.CommonValidList;
import vip.xiaonuo.biz.modular.bizuservideo.entity.BizUserVideo;
import vip.xiaonuo.biz.modular.bizuservideo.param.BizUserVideoAddParam;
import vip.xiaonuo.biz.modular.bizuservideo.param.BizUserVideoEditParam;
import vip.xiaonuo.biz.modular.bizuservideo.param.BizUserVideoIdParam;
import vip.xiaonuo.biz.modular.bizuservideo.param.BizUserVideoPageParam;
import vip.xiaonuo.biz.modular.bizuservideo.service.BizUserVideoService;

import javax.annotation.Resource;
import javax.validation.Valid;
import javax.validation.constraints.NotEmpty;
import java.util.List;

/**
 * 用户视频关联表控制器
 *
 * @author fanzherong
 * @date  2025/02/17 16:03
 */
@Api(tags = "用户视频关联表控制器")
@ApiSupport(author = "SNOWY_TEAM", order = 1)
@RestController
@Validated
public class BizUserVideoController {

    @Resource
    private BizUserVideoService bizUserVideoService;

    /**
     * 获取用户视频关联表分页
     *
     * @author fanzherong
     * @date  2025/02/17 16:03
     */
    @ApiOperationSupport(order = 1)
    @ApiOperation("获取用户视频关联表分页")
    @SaCheckPermission("/biz/bizuservideo/page")
    @GetMapping("/biz/bizuservideo/page")
    public CommonResult<Page<BizUserVideo>> page(BizUserVideoPageParam bizUserVideoPageParam) {
        return CommonResult.data(bizUserVideoService.page(bizUserVideoPageParam));
    }

    /**
     * 添加用户视频关联表
     *
     * @author fanzherong
     * @date  2025/02/17 16:03
     */
    @ApiOperationSupport(order = 2)
    @ApiOperation("添加用户视频关联表")
    @CommonLog("添加用户视频关联表")
    @SaCheckPermission("/biz/bizuservideo/add")
    @PostMapping("/biz/bizuservideo/add")
    public CommonResult<String> add(@RequestBody @Valid BizUserVideoAddParam bizUserVideoAddParam) {
        bizUserVideoService.add(bizUserVideoAddParam);
        return CommonResult.ok();
    }

    /**
     * 编辑用户视频关联表
     *
     * @author fanzherong
     * @date  2025/02/17 16:03
     */
    @ApiOperationSupport(order = 3)
    @ApiOperation("编辑用户视频关联表")
    @CommonLog("编辑用户视频关联表")
    @SaCheckPermission("/biz/bizuservideo/edit")
    @PostMapping("/biz/bizuservideo/edit")
    public CommonResult<String> edit(@RequestBody @Valid BizUserVideoEditParam bizUserVideoEditParam) {
        bizUserVideoService.edit(bizUserVideoEditParam);
        return CommonResult.ok();
    }

    /**
     * 删除用户视频关联表
     *
     * @author fanzherong
     * @date  2025/02/17 16:03
     */
    @ApiOperationSupport(order = 4)
    @ApiOperation("删除用户视频关联表")
    @CommonLog("删除用户视频关联表")
    @SaCheckPermission("/biz/bizuservideo/delete")
    @PostMapping("/biz/bizuservideo/delete")
    public CommonResult<String> delete(@RequestBody @Valid @NotEmpty(message = "集合不能为空")
                                                   CommonValidList<BizUserVideoIdParam> bizUserVideoIdParamList) {
        bizUserVideoService.delete(bizUserVideoIdParamList);
        return CommonResult.ok();
    }

    /**
     * 获取用户视频关联表详情
     *
     * @author fanzherong
     * @date  2025/02/17 16:03
     */
    @ApiOperationSupport(order = 5)
    @ApiOperation("获取用户视频关联表详情")
    @SaCheckPermission("/biz/bizuservideo/detail")
    @GetMapping("/biz/bizuservideo/detail")
    public CommonResult<BizUserVideo> detail(@Valid BizUserVideoIdParam bizUserVideoIdParam) {
        return CommonResult.data(bizUserVideoService.detail(bizUserVideoIdParam));
    }

    /**
     * 根据视频id查询用户列表
     * @param bizUserVideoPageParam
     * @return
     */
    @ApiOperationSupport(order = 5)
    @ApiOperation("根据视频id查询用户列表")
    @SaCheckPermission("/biz/bizuservideo/list")
    @GetMapping("/biz/bizuservideo/list")
    public CommonResult<List<BizUserVideo>> list(BizUserVideoPageParam bizUserVideoPageParam) {
        return CommonResult.data(bizUserVideoService.list(bizUserVideoPageParam));
    }
}
