/*
 * Copyright [2022] [https://www.xiaonuo.vip]
 *
 * Snowy采用APACHE LICENSE 2.0开源协议，您在使用过程中，需要注意以下几点：
 *
 * 1.请不要删除和修改根目录下的LICENSE文件。
 * 2.请不要删除和修改Snowy源码头部的版权声明。
 * 3.本项目代码可免费商业使用，商业使用请保留源码和相关描述文件的项目出处，作者声明等。
 * 4.分发源码时候，请注明软件出处 https://www.xiaonuo.vip
 * 5.不可二次分发开源参与同类竞品，如有想法可联系团队xiaonuobase@qq.com商议合作。
 * 6.若您的项目无法满足以上几点，需要更多功能代码，获取Snowy商业授权许可，请在官网购买授权，地址为 https://www.xiaonuo.vip
 */
package vip.xiaonuo.biz.modular.bizvideo.controller;

import cn.dev33.satoken.annotation.SaCheckPermission;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.github.xiaoymin.knife4j.annotations.ApiOperationSupport;
import com.github.xiaoymin.knife4j.annotations.ApiSupport;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.*;
import vip.xiaonuo.common.annotation.CommonLog;
import vip.xiaonuo.common.pojo.CommonResult;
import vip.xiaonuo.common.pojo.CommonValidList;
import vip.xiaonuo.biz.modular.bizvideo.entity.BizVideo;
import vip.xiaonuo.biz.modular.bizvideo.param.BizVideoAddParam;
import vip.xiaonuo.biz.modular.bizvideo.param.BizVideoEditParam;
import vip.xiaonuo.biz.modular.bizvideo.param.BizVideoIdParam;
import vip.xiaonuo.biz.modular.bizvideo.param.BizVideoPageParam;
import vip.xiaonuo.biz.modular.bizvideo.service.BizVideoService;

import javax.annotation.Resource;
import javax.validation.Valid;
import javax.validation.constraints.NotEmpty;
import java.util.Map;

/**
 * 视频配置控制器
 *
 * @author fanzherong
 * @date  2025/02/17 10:05
 */
@Api(tags = "视频配置控制器")
@ApiSupport(author = "SNOWY_TEAM", order = 1)
@RestController
@Validated
public class BizVideoController {

    @Resource
    private BizVideoService bizVideoService;

    /**
     * 获取视频配置分页
     *
     * @author fanzherong
     * @date  2025/02/17 10:05
     */
    @ApiOperationSupport(order = 1)
    @ApiOperation("获取视频配置分页")
    @SaCheckPermission("/biz/bizvideo/page")
    @GetMapping("/biz/bizvideo/page")
    public CommonResult<Page<BizVideo>> page(BizVideoPageParam bizVideoPageParam) {
        return CommonResult.data(bizVideoService.page(bizVideoPageParam));
    }

    /**
     * 小程序获取用户视频列表
     * @param bizVideoPageParam
     * @return
     */
    @ApiOperationSupport(order = 7)
    @ApiOperation("小程序获取用户视频列表")
    @SaCheckPermission("/api/mini/biz/bizvideo/queryVideo")
    @GetMapping("/api/mini/biz/bizvideo/queryVideo")
    public CommonResult<Page<BizVideo>> queryVideo(BizVideoPageParam bizVideoPageParam) {
        return CommonResult.data(bizVideoService.queryVideo(bizVideoPageParam));
    }

    /**
     * 添加视频配置
     *
     * @author fanzherong
     * @date  2025/02/17 10:05
     */
    @ApiOperationSupport(order = 2)
    @ApiOperation("添加视频配置")
    @CommonLog("添加视频配置")
    @SaCheckPermission("/biz/bizvideo/add")
    @PostMapping("/biz/bizvideo/add")
    public CommonResult<String> add(@RequestBody @Valid BizVideoAddParam bizVideoAddParam) {
        bizVideoService.add(bizVideoAddParam);
        return CommonResult.ok();
    }

    /**
     * 编辑视频配置
     *
     * @author fanzherong
     * @date  2025/02/17 10:05
     */
    @ApiOperationSupport(order = 3)
    @ApiOperation("编辑视频配置")
    @CommonLog("编辑视频配置")
    @SaCheckPermission("/biz/bizvideo/edit")
    @PostMapping("/biz/bizvideo/edit")
    public CommonResult<String> edit(@RequestBody @Valid BizVideoEditParam bizVideoEditParam) {
        bizVideoService.edit(bizVideoEditParam);
        return CommonResult.ok();
    }

    /**
     * 删除视频配置
     *
     * @author fanzherong
     * @date  2025/02/17 10:05
     */
    @ApiOperationSupport(order = 4)
    @ApiOperation("删除视频配置")
    @CommonLog("删除视频配置")
    @SaCheckPermission("/biz/bizvideo/delete")
    @PostMapping("/biz/bizvideo/delete")
    public CommonResult<String> delete(@RequestBody @Valid @NotEmpty(message = "集合不能为空")
                                                   CommonValidList<BizVideoIdParam> bizVideoIdParamList) {
        bizVideoService.delete(bizVideoIdParamList);
        return CommonResult.ok();
    }

    /**
     * 获取视频配置详情
     *
     * @author fanzherong
     * @date  2025/02/17 10:05
     */
    @ApiOperationSupport(order = 5)
    @ApiOperation("获取视频配置详情")
    @SaCheckPermission("/biz/bizvideo/detail")
    @GetMapping("/biz/bizvideo/detail")
    public CommonResult<BizVideo> detail(@Valid BizVideoIdParam bizVideoIdParam) {
        return CommonResult.data(bizVideoService.detail(bizVideoIdParam));
    }

    /**
     * 获取key
     * @param
     * @return
     */
    @ApiOperationSupport(order = 5)
    @ApiOperation("获取视频配置详情")
    @SaCheckPermission("/api/mini/biz/bizvideo/getAppkey")
    @GetMapping("/api/mini/biz/bizvideo/getAppkey")
    public CommonResult<Map<String,Object>> getAppkey() {
        return CommonResult.data(bizVideoService.getAppkey());
    }

    /**
     * 获取视频播放地址
     * @return
     */
    @ApiOperationSupport(order = 5)
    @ApiOperation("获取视频播放地址")
    @SaCheckPermission("/biz/bizvideo/getVideoUrl")
    @GetMapping("/biz/bizvideo/getVideoUrl")
    public CommonResult<Map<String,Object>> getVideoUrl(BizVideoIdParam bizVideoIdParam) {
        return CommonResult.data(bizVideoService.getVideoUrl(bizVideoIdParam));
    }

    /**
     * 小程序获取视频播放地址
     * @return
     */
    @ApiOperationSupport(order = 5)
    @ApiOperation("小程序获取视频播放地址")
    @SaCheckPermission("/api/mini/biz/bizvideo/getVideoUrlByProgram")
    @GetMapping("/api/mini/biz/bizvideo/getVideoUrlByProgram")
    public CommonResult<Map<String,Object>> getVideoUrlByProgram(BizVideoIdParam bizVideoIdParam) {
        return CommonResult.data(bizVideoService.getVideoUrl(bizVideoIdParam));
    }
}
