/*
 * Copyright [2022] [https://www.xiaonuo.vip]
 *
 * Snowy采用APACHE LICENSE 2.0开源协议，您在使用过程中，需要注意以下几点：
 *
 * 1.请不要删除和修改根目录下的LICENSE文件。
 * 2.请不要删除和修改Snowy源码头部的版权声明。
 * 3.本项目代码可免费商业使用，商业使用请保留源码和相关描述文件的项目出处，作者声明等。
 * 4.分发源码时候，请注明软件出处 https://www.xiaonuo.vip
 * 5.不可二次分发开源参与同类竞品，如有想法可联系团队xiaonuobase@qq.com商议合作。
 * 6.若您的项目无法满足以上几点，需要更多功能代码，获取Snowy商业授权许可，请在官网购买授权，地址为 https://www.xiaonuo.vip
 */
package vip.xiaonuo.biz.modular.depotAccessRecord.controller;

import cn.dev33.satoken.annotation.SaCheckPermission;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.github.xiaoymin.knife4j.annotations.ApiOperationSupport;
import com.github.xiaoymin.knife4j.annotations.ApiSupport;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RestController;
import vip.xiaonuo.common.annotation.CommonLog;
import vip.xiaonuo.common.pojo.CommonResult;
import vip.xiaonuo.common.pojo.CommonValidList;
import vip.xiaonuo.biz.modular.depotAccessRecord.entity.DepotAccessRecord;
import vip.xiaonuo.biz.modular.depotAccessRecord.param.DepotAccessRecordAddParam;
import vip.xiaonuo.biz.modular.depotAccessRecord.param.DepotAccessRecordEditParam;
import vip.xiaonuo.biz.modular.depotAccessRecord.param.DepotAccessRecordIdParam;
import vip.xiaonuo.biz.modular.depotAccessRecord.param.DepotAccessRecordPageParam;
import vip.xiaonuo.biz.modular.depotAccessRecord.service.DepotAccessRecordService;

import javax.annotation.Resource;
import javax.validation.Valid;
import javax.validation.constraints.NotEmpty;

/**
 * 进出场记录表控制器
 *
 * @author xiang
 * @date 2023/10/23 16:28
 */
@Api(tags = "进出场记录表控制器")
@ApiSupport(author = "SNOWY_TEAM", order = 1)
@RestController
@Validated
public class DepotAccessRecordController {

    @Resource
    private DepotAccessRecordService depotAccessRecordService;

    /**
     * 获取进出场记录表分页
     *
     * @author xiang
     * @date 2023/10/23 16:28
     */
    @ApiOperationSupport(order = 1)
    @ApiOperation("获取进出场记录表分页")
    @SaCheckPermission("/biz/depotAccessRecord/page")
    @GetMapping("/biz/depotAccessRecord/page")
    public CommonResult<Page<DepotAccessRecord>> page(DepotAccessRecordPageParam depotAccessRecordPageParam) {
        return CommonResult.data(depotAccessRecordService.page(depotAccessRecordPageParam));
    }

    /**
     * 添加进出场记录表
     *
     * @author xiang
     * @date 2023/10/23 16:28
     */
    @ApiOperationSupport(order = 2)
    @ApiOperation("添加进出场记录表")
    @CommonLog("添加进出场记录表")
    @SaCheckPermission("/biz/depotAccessRecord/add")
    @PostMapping("/biz/depotAccessRecord/add")
    public CommonResult<String> add(@RequestBody @Valid DepotAccessRecordAddParam depotAccessRecordAddParam) {
        depotAccessRecordService.add(depotAccessRecordAddParam);
        return CommonResult.ok();
    }

    /**
     * 编辑进出场记录表
     *
     * @author xiang
     * @date 2023/10/23 16:28
     */
    @ApiOperationSupport(order = 3)
    @ApiOperation("编辑进出场记录表")
    @CommonLog("编辑进出场记录表")
    @SaCheckPermission("/biz/depotAccessRecord/edit")
    @PostMapping("/biz/depotAccessRecord/edit")
    public CommonResult<String> edit(@RequestBody @Valid DepotAccessRecordEditParam depotAccessRecordEditParam) {
        depotAccessRecordService.edit(depotAccessRecordEditParam);
        return CommonResult.ok();
    }

    /**
     * 删除进出场记录表
     *
     * @author xiang
     * @date 2023/10/23 16:28
     */
    @ApiOperationSupport(order = 4)
    @ApiOperation("删除进出场记录表")
    @CommonLog("删除进出场记录表")
    @SaCheckPermission("/biz/depotAccessRecord/delete")
    @PostMapping("/biz/depotAccessRecord/delete")
    public CommonResult<String> delete(@RequestBody @Valid @NotEmpty(message = "集合不能为空")
                                               CommonValidList<DepotAccessRecordIdParam> depotAccessRecordIdParamList) {
        depotAccessRecordService.delete(depotAccessRecordIdParamList);
        return CommonResult.ok();
    }

    /**
     * 获取进出场记录表详情
     *
     * @author xiang
     * @date 2023/10/23 16:28
     */
    @ApiOperationSupport(order = 5)
    @ApiOperation("获取进出场记录表详情")
    @SaCheckPermission("/biz/depotAccessRecord/detail")
    @GetMapping("/biz/depotAccessRecord/detail")
    public CommonResult<DepotAccessRecord> detail(@Valid DepotAccessRecordIdParam depotAccessRecordIdParam) {
        return CommonResult.data(depotAccessRecordService.detail(depotAccessRecordIdParam));
    }
}
