/*
 * Copyright [2022] [https://www.xiaonuo.vip]
 *
 * Snowy采用APACHE LICENSE 2.0开源协议，您在使用过程中，需要注意以下几点：
 *
 * 1.请不要删除和修改根目录下的LICENSE文件。
 * 2.请不要删除和修改Snowy源码头部的版权声明。
 * 3.本项目代码可免费商业使用，商业使用请保留源码和相关描述文件的项目出处，作者声明等。
 * 4.分发源码时候，请注明软件出处 https://www.xiaonuo.vip
 * 5.不可二次分发开源参与同类竞品，如有想法可联系团队xiaonuobase@qq.com商议合作。
 * 6.若您的项目无法满足以上几点，需要更多功能代码，获取Snowy商业授权许可，请在官网购买授权，地址为 https://www.xiaonuo.vip
 */
package vip.xiaonuo.biz.modular.depotAlarm.controller;

import cn.dev33.satoken.annotation.SaCheckPermission;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.github.xiaoymin.knife4j.annotations.ApiOperationSupport;
import com.github.xiaoymin.knife4j.annotations.ApiSupport;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RestController;
import vip.xiaonuo.biz.modular.depotAlarm.entity.DepotAlarmAddCountGroupTypeVo;
import vip.xiaonuo.biz.modular.depotAlarm.param.*;
import vip.xiaonuo.common.annotation.CommonLog;
import vip.xiaonuo.common.pojo.CommonResult;
import vip.xiaonuo.common.pojo.CommonValidList;
import vip.xiaonuo.biz.modular.depotAlarm.entity.DepotAlarm;
import vip.xiaonuo.biz.modular.depotAlarm.service.DepotAlarmService;

import javax.annotation.Resource;
import javax.validation.Valid;
import javax.validation.constraints.NotEmpty;

/**
 * 异常报警控制器
 *
 * @author xiang
 * @date 2023/09/19 16:23
 */
@Api(tags = "异常报警控制器")
@ApiSupport(author = "SNOWY_TEAM", order = 1)
@RestController
@Validated
public class DepotAlarmController {

    @Resource
    private DepotAlarmService depotAlarmService;

    /**
     * 获取异常报警分页
     *
     * @author xiang
     * @date 2023/09/19 16:23
     */
    @ApiOperationSupport(order = 1)
    @ApiOperation("获取异常报警分页")
    @SaCheckPermission("/biz/depotAlarm/page")
    @GetMapping("/biz/depotAlarm/page")
    public CommonResult<Page<DepotAlarm>> page(DepotAlarmPageParam depotAlarmPageParam) {
        return CommonResult.data(depotAlarmService.page(depotAlarmPageParam));
    }

    /**
     * 添加异常报警
     *
     * @author xiang
     * @date 2023/09/19 16:23
     */
    @ApiOperationSupport(order = 2)
    @ApiOperation("添加异常报警")
    @CommonLog("添加异常报警")
    @SaCheckPermission("/biz/depotAlarm/add")
    @PostMapping("/biz/depotAlarm/add")
    public CommonResult<String> add(@RequestBody @Valid DepotAlarmAddParam depotAlarmAddParam) {
        depotAlarmService.add(depotAlarmAddParam);
        return CommonResult.ok();
    }

    /**
     * 编辑异常报警
     *
     * @author xiang
     * @date 2023/09/19 16:23
     */
    @ApiOperationSupport(order = 3)
    @ApiOperation("编辑异常报警")
    @CommonLog("编辑异常报警")
    @SaCheckPermission("/biz/depotAlarm/edit")
    @PostMapping("/biz/depotAlarm/edit")
    public CommonResult<String> edit(@RequestBody @Valid DepotAlarmEditParam depotAlarmEditParam) {
        depotAlarmService.edit(depotAlarmEditParam);
        return CommonResult.ok();
    }

    /**
     * 删除异常报警
     *
     * @author xiang
     * @date 2023/09/19 16:23
     */
    @ApiOperationSupport(order = 4)
    @ApiOperation("删除异常报警")
    @CommonLog("删除异常报警")
    @SaCheckPermission("/biz/depotAlarm/delete")
    @PostMapping("/biz/depotAlarm/delete")
    public CommonResult<String> delete(@RequestBody @Valid @NotEmpty(message = "集合不能为空")
                                               CommonValidList<DepotAlarmIdParam> depotAlarmIdParamList) {
        depotAlarmService.delete(depotAlarmIdParamList);
        return CommonResult.ok();
    }

    /**
     * 获取异常报警详情
     *
     * @author xiang
     * @date 2023/09/19 16:23
     */
    @ApiOperationSupport(order = 5)
    @ApiOperation("获取异常报警详情")
    @SaCheckPermission("/biz/depotAlarm/detail")
    @GetMapping("/biz/depotAlarm/detail")
    public CommonResult<DepotAlarm> detail(@Valid DepotAlarmIdParam depotAlarmIdParam) {
        return CommonResult.data(depotAlarmService.detail(depotAlarmIdParam));
    }

    /**
     * 获取异常待处理统计数据
     *
     * @author xiang
     * @date 2023/09/19 16:23
     */
    @ApiOperationSupport(order = 1)
    @ApiOperation("获取异常待处理统计数据")
    @GetMapping("/biz/depotAlarm/getAlarmAddCountGroupType")
    public CommonResult<DepotAlarmAddCountGroupTypeVo> getAlarmAddCountGroupType(DepotAlarmPageParam depotAlarmPageParam) {
        return CommonResult.data(depotAlarmService.getAlarmAddCountGroupType(depotAlarmPageParam));
    }

}
