/*
 * Copyright [2022] [https://www.xiaonuo.vip]
 *
 * Snowy采用APACHE LICENSE 2.0开源协议，您在使用过程中，需要注意以下几点：
 *
 * 1.请不要删除和修改根目录下的LICENSE文件。
 * 2.请不要删除和修改Snowy源码头部的版权声明。
 * 3.本项目代码可免费商业使用，商业使用请保留源码和相关描述文件的项目出处，作者声明等。
 * 4.分发源码时候，请注明软件出处 https://www.xiaonuo.vip
 * 5.不可二次分发开源参与同类竞品，如有想法可联系团队xiaonuobase@qq.com商议合作。
 * 6.若您的项目无法满足以上几点，需要更多功能代码，获取Snowy商业授权许可，请在官网购买授权，地址为 https://www.xiaonuo.vip
 */
package vip.xiaonuo.biz.modular.depotDevice.controller;

import cn.dev33.satoken.annotation.SaCheckPermission;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.github.xiaoymin.knife4j.annotations.ApiOperationSupport;
import com.github.xiaoymin.knife4j.annotations.ApiSupport;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RestController;
import vip.xiaonuo.biz.modular.depotDevice.param.DepotDeviceEditParam;
import vip.xiaonuo.common.annotation.CommonLog;
import vip.xiaonuo.common.pojo.CommonResult;
import vip.xiaonuo.biz.modular.depotDevice.entity.DepotDevice;
import vip.xiaonuo.biz.modular.depotDevice.param.DepotDeviceIdParam;
import vip.xiaonuo.biz.modular.depotDevice.param.DepotDevicePageParam;
import vip.xiaonuo.biz.modular.depotDevice.service.DepotDeviceService;

import javax.annotation.Resource;
import javax.validation.Valid;

/**
 * 设备列表控制器
 *
 * @author xiang
 * @date 2023/09/19 10:12
 */
@Api(tags = "设备列表控制器")
@ApiSupport(author = "SNOWY_TEAM", order = 1)
@RestController
@Validated
public class DepotDeviceController {

    @Resource
    private DepotDeviceService depotDeviceService;

    /**
     * 获取设备列表分页
     *
     * @author xiang
     * @date 2023/09/19 10:12
     */
    @ApiOperationSupport(order = 1)
    @ApiOperation("获取设备列表分页")
    @SaCheckPermission("/biz/depotDevice/page")
    @GetMapping("/biz/depotDevice/page")
    public CommonResult<Page<DepotDevice>> page(DepotDevicePageParam depotDevicePageParam) {
        return CommonResult.data(depotDeviceService.page(depotDevicePageParam));
    }

    /**
     * 编辑设备
     *
     * @author xiang
     * @date 2023/09/19 16:23
     */
    @ApiOperationSupport(order = 3)
    @ApiOperation("编辑设备")
    @CommonLog("编辑设备")
    @PostMapping("/biz/depotDevice/edit")
    public CommonResult<String> edit(@RequestBody @Valid DepotDeviceEditParam param) {
        depotDeviceService.edit(param);
        return CommonResult.ok();
    }

    /**
     * 获取设备列表详情
     *
     * @author xiang
     * @date 2023/09/19 10:12
     */
    @ApiOperationSupport(order = 5)
    @ApiOperation("获取设备列表详情")
    @SaCheckPermission("/biz/depotDevice/detail")
    @GetMapping("/biz/depotDevice/detail")
    public CommonResult<DepotDevice> detail(@Valid DepotDeviceIdParam depotDeviceIdParam) {
        return CommonResult.data(depotDeviceService.detail(depotDeviceIdParam));
    }

}
