/*
 * Copyright [2022] [https://www.xiaonuo.vip]
 *
 * Snowy采用APACHE LICENSE 2.0开源协议，您在使用过程中，需要注意以下几点：
 *
 * 1.请不要删除和修改根目录下的LICENSE文件。
 * 2.请不要删除和修改Snowy源码头部的版权声明。
 * 3.本项目代码可免费商业使用，商业使用请保留源码和相关描述文件的项目出处，作者声明等。
 * 4.分发源码时候，请注明软件出处 https://www.xiaonuo.vip
 * 5.不可二次分发开源参与同类竞品，如有想法可联系团队xiaonuobase@qq.com商议合作。
 * 6.若您的项目无法满足以上几点，需要更多功能代码，获取Snowy商业授权许可，请在官网购买授权，地址为 https://www.xiaonuo.vip
 */
package vip.xiaonuo.biz.modular.depotFlyerSiteLogWarn.controller;

import cn.dev33.satoken.annotation.SaCheckPermission;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.github.xiaoymin.knife4j.annotations.ApiOperationSupport;
import com.github.xiaoymin.knife4j.annotations.ApiSupport;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RestController;
import vip.xiaonuo.biz.core.Const.BizStr;
import vip.xiaonuo.biz.modular.depotFlyerSiteLog.entity.DepotFlyerSiteLog;
import vip.xiaonuo.biz.modular.depotFlyerSiteLog.param.DepotFlyerSiteLogAddParam;
import vip.xiaonuo.biz.modular.depotFlyerSiteLog.param.DepotFlyerSiteLogEditParam;
import vip.xiaonuo.biz.modular.depotFlyerSiteLog.param.DepotFlyerSiteLogIdParam;
import vip.xiaonuo.biz.modular.depotFlyerSiteLog.param.DepotFlyerSiteLogPageParam;
import vip.xiaonuo.common.annotation.CommonLog;
import vip.xiaonuo.common.pojo.CommonResult;
import vip.xiaonuo.common.pojo.CommonValidList;
import vip.xiaonuo.biz.modular.depotFlyerSiteLogWarn.service.DepotFlyerSiteLogWarnService;

import javax.annotation.Resource;
import javax.validation.Valid;
import javax.validation.constraints.NotEmpty;

/**
 * 飞行监管异常日志控制器
 *
 * @author xiang
 * @date 2023/09/19 14:22
 */
@Api(tags = "飞行监管异常日志控制器")
@ApiSupport(author = "SNOWY_TEAM", order = 1)
@RestController
@Validated
public class DepotFlyerSiteLogWarnController {

    @Resource
    private DepotFlyerSiteLogWarnService depotFlyerSiteLogWarnService;

    /**
     * 获取飞行监管异常日志分页
     *
     * @author xiang
     * @date 2023/09/19 14:22
     */
    @ApiOperationSupport(order = 1)
    @ApiOperation("获取飞行监管异常日志分页")
    @SaCheckPermission("/biz/depotFlyerSiteLogWarn/page")
    @GetMapping("/biz/depotFlyerSiteLogWarn/page")
    public CommonResult<Page<DepotFlyerSiteLog>> page(DepotFlyerSiteLogPageParam param) {
        param.setAlarmFlag(BizStr.FLAG_SHI);
        return CommonResult.data(depotFlyerSiteLogWarnService.page(param));
    }

    /**
     * 添加飞行监管异常日志
     *
     * @author xiang
     * @date 2023/09/19 14:22
     */
    @ApiOperationSupport(order = 2)
    @ApiOperation("添加飞行监管异常日志")
    @CommonLog("添加飞行监管异常日志")
    @SaCheckPermission("/biz/depotFlyerSiteLogWarn/add")
    @PostMapping("/biz/depotFlyerSiteLogWarn/add")
    public CommonResult<String> add(@RequestBody @Valid DepotFlyerSiteLogAddParam depotFlyerSiteLogWarnAddParam) {
        depotFlyerSiteLogWarnService.add(depotFlyerSiteLogWarnAddParam);
        return CommonResult.ok();
    }

    /**
     * 编辑飞行监管异常日志
     *
     * @author xiang
     * @date 2023/09/19 14:22
     */
    @ApiOperationSupport(order = 3)
    @ApiOperation("编辑飞行监管异常日志")
    @CommonLog("编辑飞行监管异常日志")
    @SaCheckPermission("/biz/depotFlyerSiteLogWarn/edit")
    @PostMapping("/biz/depotFlyerSiteLogWarn/edit")
    public CommonResult<String> edit(@RequestBody @Valid DepotFlyerSiteLogEditParam depotFlyerSiteLogWarnEditParam) {
        depotFlyerSiteLogWarnService.edit(depotFlyerSiteLogWarnEditParam);
        return CommonResult.ok();
    }

    /**
     * 删除飞行监管异常日志
     *
     * @author xiang
     * @date 2023/09/19 14:22
     */
    @ApiOperationSupport(order = 4)
    @ApiOperation("删除飞行监管异常日志")
    @CommonLog("删除飞行监管异常日志")
    @SaCheckPermission("/biz/depotFlyerSiteLogWarn/delete")
    @PostMapping("/biz/depotFlyerSiteLogWarn/delete")
    public CommonResult<String> delete(@RequestBody @Valid @NotEmpty(message = "集合不能为空")
                                               CommonValidList<DepotFlyerSiteLogIdParam> depotFlyerSiteLogWarnIdParamList) {
        depotFlyerSiteLogWarnService.delete(depotFlyerSiteLogWarnIdParamList);
        return CommonResult.ok();
    }

    /**
     * 获取飞行监管异常日志详情
     *
     * @author xiang
     * @date 2023/09/19 14:22
     */
    @ApiOperationSupport(order = 5)
    @ApiOperation("获取飞行监管异常日志详情")
    @SaCheckPermission("/biz/depotFlyerSiteLogWarn/detail")
    @GetMapping("/biz/depotFlyerSiteLogWarn/detail")
    public CommonResult<DepotFlyerSiteLog> detail(@Valid DepotFlyerSiteLogIdParam depotFlyerSiteLogWarnIdParam) {
        return CommonResult.data(depotFlyerSiteLogWarnService.detail(depotFlyerSiteLogWarnIdParam));
    }
}
