/*
 * Copyright [2022] [https://www.xiaonuo.vip]
 *
 * Snowy采用APACHE LICENSE 2.0开源协议，您在使用过程中，需要注意以下几点：
 *
 * 1.请不要删除和修改根目录下的LICENSE文件。
 * 2.请不要删除和修改Snowy源码头部的版权声明。
 * 3.本项目代码可免费商业使用，商业使用请保留源码和相关描述文件的项目出处，作者声明等。
 * 4.分发源码时候，请注明软件出处 https://www.xiaonuo.vip
 * 5.不可二次分发开源参与同类竞品，如有想法可联系团队xiaonuobase@qq.com商议合作。
 * 6.若您的项目无法满足以上几点，需要更多功能代码，获取Snowy商业授权许可，请在官网购买授权，地址为 https://www.xiaonuo.vip
 */
package vip.xiaonuo.biz.modular.depotGrainInfo.controller;

import cn.dev33.satoken.annotation.SaCheckPermission;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.github.xiaoymin.knife4j.annotations.ApiOperationSupport;
import com.github.xiaoymin.knife4j.annotations.ApiSupport;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.*;
import org.springframework.web.multipart.MultipartFile;
import vip.xiaonuo.biz.modular.depotGrainInfo.param.*;
import vip.xiaonuo.common.annotation.CommonLog;
import vip.xiaonuo.common.pojo.CommonResult;
import vip.xiaonuo.common.pojo.CommonValidList;
import vip.xiaonuo.biz.modular.depotGrainInfo.entity.DepotGrainInfo;
import vip.xiaonuo.biz.modular.depotGrainInfo.service.DepotGrainInfoService;
import vip.xiaonuo.dev.modular.file.param.DevFileIdParam;
import vip.xiaonuo.dev.modular.file.service.DevFileService;

import javax.annotation.Resource;
import javax.validation.Valid;
import javax.validation.constraints.NotEmpty;

/**
 * 企业客户信息表控制器
 *
 * @author xiang
 * @date 2023/09/14 15:00
 */
@Api(tags = "企业客户信息表控制器")
@ApiSupport(author = "SNOWY_TEAM", order = 1)
@RestController
@Validated
public class DepotGrainInfoController {

    @Resource
    private DevFileService devFileService;
    @Resource
    private DepotGrainInfoService depotGrainInfoService;

    /**
     * 获取企业客户信息表分页
     *
     * @author xiang
     * @date 2023/09/14 15:00
     */
    @ApiOperationSupport(order = 1)
    @ApiOperation("获取企业客户信息表分页")
    @SaCheckPermission("/biz/depotGrainInfo/page")
    @GetMapping("/biz/depotGrainInfo/page")
    public CommonResult<Page<DepotGrainInfo>> page(DepotGrainInfoPageParam depotGrainInfoPageParam) {
        return CommonResult.data(depotGrainInfoService.page(depotGrainInfoPageParam));
    }

    /**
     * 添加企业客户信息表
     *
     * @author xiang
     * @date 2023/09/14 15:00
     */
    @ApiOperationSupport(order = 2)
    @ApiOperation("添加企业客户信息表")
    @CommonLog("添加企业客户信息表")
    @SaCheckPermission("/biz/depotGrainInfo/add")
    @PostMapping("/biz/depotGrainInfo/add")
    public CommonResult<String> add(@RequestBody @Valid DepotGrainInfoAddParam depotGrainInfoAddParam) {
        depotGrainInfoService.add(depotGrainInfoAddParam);
        return CommonResult.ok();
    }

    /**
     * 编辑企业客户信息表
     *
     * @author xiang
     * @date 2023/09/14 15:00
     */
    @ApiOperationSupport(order = 3)
    @ApiOperation("编辑企业客户信息表")
    @CommonLog("编辑企业客户信息表")
    @SaCheckPermission("/biz/depotGrainInfo/edit")
    @PostMapping("/biz/depotGrainInfo/edit")
    public CommonResult<String> edit(@RequestBody @Valid DepotGrainInfoEditParam depotGrainInfoEditParam) {
        depotGrainInfoService.edit(depotGrainInfoEditParam);
        return CommonResult.ok();
    }

    /**
     * 删除企业客户信息表
     *
     * @author xiang
     * @date 2023/09/14 15:00
     */
    @ApiOperationSupport(order = 4)
    @ApiOperation("删除企业客户信息表")
    @CommonLog("删除企业客户信息表")
    @SaCheckPermission("/biz/depotGrainInfo/delete")
    @PostMapping("/biz/depotGrainInfo/delete")
    public CommonResult<String> delete(@RequestBody @Valid @NotEmpty(message = "集合不能为空")
                                               CommonValidList<DepotGrainInfoIdParam> depotGrainInfoIdParamList) {
        depotGrainInfoService.delete(depotGrainInfoIdParamList);
        return CommonResult.ok();
    }

    /**
     * 获取企业客户信息表详情
     *
     * @author xiang
     * @date 2023/09/14 15:00
     */
    @ApiOperationSupport(order = 5)
    @ApiOperation("获取企业客户信息表详情")
    @SaCheckPermission("/biz/depotGrainInfo/detail")
    @GetMapping("/biz/depotGrainInfo/detail")
    public CommonResult<Object> detail(@Valid DepotGrainInfoIdParam param) {
        return CommonResult.data(depotGrainInfoService.detail(param));
    }

    /**
     * 企业客户上传文件
     *
     * @author xiang
     * @date 2023/09/14 15:00
     */
    @ApiOperationSupport(order = 7)
    @ApiOperation("企业客户上传文件")
    @SaCheckPermission("/biz/depotGrainInfo/fileUpload")
    @PostMapping("/biz/depotGrainInfo/fileUpload")
    public CommonResult<Object> fileUpload(@RequestPart("file") MultipartFile file
            , @RequestPart("id") String id) {
        depotGrainInfoService.fileUpload(file, id);
        return CommonResult.ok();
    }

    /**
     * 获取企业客户上传文件
     *
     * @author xiang
     * @date 2023/09/14 15:00
     */
    @ApiOperationSupport(order = 6)
    @ApiOperation("获取企业客户上传文件")
    @SaCheckPermission("/biz/depotGrainInfo/fileLst")
    @GetMapping("/biz/depotGrainInfo/fileLst")
    public CommonResult<Object> fileLst(@Valid DepotGrainInfoFileParam fileParam) {
        return CommonResult.data(depotGrainInfoService.fileLst(fileParam.getId()));
    }

    /**
     * 获取企业客户上传文件
     *
     * @author xiang
     * @date 2023/09/14 15:00
     */
    @ApiOperationSupport(order = 6)
    @ApiOperation("获取企业客户上传文件")
    @SaCheckPermission("/biz/depotGrainInfo/fileDetail")
    @GetMapping("/biz/depotGrainInfo/fileDetail")
    public CommonResult<Object> fileDetail(@Valid DevFileIdParam param) {
        return CommonResult.data(devFileService.detail(param));
    }

    /**
     * 删除企业客户上传文件
     *
     * @author xiang
     * @date 2023/09/14 15:00
     */
    @ApiOperationSupport(order = 6)
    @ApiOperation("删除企业客户上传文件")
    @SaCheckPermission("/biz/depotGrainInfo/fileDelete")
    @GetMapping("/biz/depotGrainInfo/fileDelete")
    public CommonResult<Object> fileDelete(@Valid DepotGrainInfoFileParam fileParam) {
        depotGrainInfoService.bindFileUpload(new DepotGrainInfoFileParam(null, fileParam.getFileId(), null));
        return CommonResult.ok();
    }

}
