/*
 * Copyright [2022] [https://www.xiaonuo.vip]
 *
 * Snowy采用APACHE LICENSE 2.0开源协议，您在使用过程中，需要注意以下几点：
 *
 * 1.请不要删除和修改根目录下的LICENSE文件。
 * 2.请不要删除和修改Snowy源码头部的版权声明。
 * 3.本项目代码可免费商业使用，商业使用请保留源码和相关描述文件的项目出处，作者声明等。
 * 4.分发源码时候，请注明软件出处 https://www.xiaonuo.vip
 * 5.不可二次分发开源参与同类竞品，如有想法可联系团队xiaonuobase@qq.com商议合作。
 * 6.若您的项目无法满足以上几点，需要更多功能代码，获取Snowy商业授权许可，请在官网购买授权，地址为 https://www.xiaonuo.vip
 */
package vip.xiaonuo.biz.modular.depotGrainInfoPrice.entity;

import com.baomidou.mybatisplus.annotation.TableId;
import com.baomidou.mybatisplus.annotation.TableName;
import io.swagger.annotations.ApiModelProperty;
import lombok.Getter;
import lombok.Setter;
import vip.xiaonuo.biz.modular.depotGrain.entity.BaseDepotGrainName;

import java.math.BigDecimal;

/**
 * 企业客户种类价格表实体
 *
 * @author xiang
 * @date  2023/10/07 10:05
 **/
@Getter
@Setter
@TableName("DEPOT_GRAIN_INFO_PRICE")
public class DepotGrainInfoPrice extends BaseDepotGrainName {

    /** id */
    @TableId
    @ApiModelProperty(value = "id", position = 1)
    private String id;

    /** 项目编码 */
    @ApiModelProperty(value = "项目编码", position = 2)
    private String depotId;

    /** 主流价格id */
    @ApiModelProperty(value = "主流价格id", position = 3)
    private String zonePriceId;

    /** 主流价格 */
    @ApiModelProperty(value = "主流价格", position = 4)
    private BigDecimal zonePrice;

    /** 粮食种类 */
    @ApiModelProperty(value = "粮食种类", position = 5)
    private String grainGroup;

    /** 质押价格 */
    @ApiModelProperty(value = "质押价格", position = 6)
    private BigDecimal loanPrice;

    /** 平均质押价格 */
    @ApiModelProperty(value = "平均质押价格", position = 7)
    private BigDecimal loanPriceAvg;

    /** 干粮价格 */
    @ApiModelProperty(value = "干粮价格", position = 8)
    private BigDecimal dryPrice;

    /** 质押率(%) */
    @ApiModelProperty(value = "质押率(%)", position = 9)
    private BigDecimal loanRatio;

    /** 潮干比 */
    @ApiModelProperty(value = "潮干比", position = 11)
    private BigDecimal dampRatio;

    /** 干粮水份(%) */
    @ApiModelProperty(value = "干粮水份(%)", position = 12)
    private BigDecimal canonRatio;

    /** 库存量(万吨) */
    @ApiModelProperty(value = "库存量(万吨)", position = 10)
    private BigDecimal loanTonW;

    /** 累计入库重量(万吨) */
    @ApiModelProperty(value = "累计入库重量(万吨)", position = 15)
    private BigDecimal realInWeightHistory;

    /** 累计出库重量(万吨) */
    @ApiModelProperty(value = "累计出库重量(万吨)", position = 16)
    private BigDecimal realOutWeightHistory;

    /** 累计贷款重量(万吨) */
    @ApiModelProperty(value = "累计贷款重量(万吨)", position = 19)
    private BigDecimal loanWeightHistory;

    /** 累计还款重量(万吨) */
    @ApiModelProperty(value = "累计还款重量(万吨)", position = 20)
    private BigDecimal repayWeightHistory;

    /** 累计入库金额  */
    @ApiModelProperty(value = "累计入库金额", position = 17)
    private BigDecimal inAmountHistory;

    /** 累计出库金额  */
    @ApiModelProperty(value = "累计出库金额", position = 17)
    private BigDecimal outAmountHistory;

    /** 累计入库干粮重量(吨) 用于计算平均质押价格 */
    @ApiModelProperty(value = "累计入库干粮重量(吨)", position = 18)
    private BigDecimal dryInWeightHistory;

    public DepotGrainInfoPrice() {
    }

    public DepotGrainInfoPrice(String depotId) {
        this.depotId = depotId;
    }

    public DepotGrainInfoPrice(String depotId, String grainGroup) {
        this.depotId = depotId;
        this.grainGroup = grainGroup;
    }

    public DepotGrainInfoPrice(String depotId, String zonePriceId, String grainGroup) {
        this.depotId = depotId;
        this.zonePriceId = zonePriceId;
        this.grainGroup = grainGroup;
    }
}
