/*
 * Copyright [2022] [https://www.xiaonuo.vip]
 *
 * Snowy采用APACHE LICENSE 2.0开源协议，您在使用过程中，需要注意以下几点：
 *
 * 1.请不要删除和修改根目录下的LICENSE文件。
 * 2.请不要删除和修改Snowy源码头部的版权声明。
 * 3.本项目代码可免费商业使用，商业使用请保留源码和相关描述文件的项目出处，作者声明等。
 * 4.分发源码时候，请注明软件出处 https://www.xiaonuo.vip
 * 5.不可二次分发开源参与同类竞品，如有想法可联系团队xiaonuobase@qq.com商议合作。
 * 6.若您的项目无法满足以上几点，需要更多功能代码，获取Snowy商业授权许可，请在官网购买授权，地址为 https://www.xiaonuo.vip
 */
package vip.xiaonuo.biz.modular.depotRepayBook.controller;

import cn.dev33.satoken.annotation.SaCheckPermission;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.github.xiaoymin.knife4j.annotations.ApiOperationSupport;
import com.github.xiaoymin.knife4j.annotations.ApiSupport;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RestController;
import vip.xiaonuo.biz.modular.depotRepayOrder.entity.DepotRepayOrder;
import vip.xiaonuo.biz.modular.depotRepayOrder.param.DepotRepayOrderAddParam;
import vip.xiaonuo.biz.modular.depotRepayOrder.param.DepotRepayOrderEditParam;
import vip.xiaonuo.biz.modular.depotRepayOrder.param.DepotRepayOrderIdParam;
import vip.xiaonuo.biz.modular.depotRepayOrder.param.DepotRepayOrderPageParam;
import vip.xiaonuo.biz.modular.depotRepayOrder.service.DepotRepayOrderService;
import vip.xiaonuo.common.annotation.CommonLog;
import vip.xiaonuo.common.pojo.CommonResult;
import vip.xiaonuo.common.pojo.CommonValidList;

import javax.annotation.Resource;
import javax.validation.Valid;
import javax.validation.constraints.NotEmpty;

/**
 * 还款提货预约控制器
 *
 * @author xiang
 * @date  2023/09/14 13:53
 */
@Api(tags = "还款提货预约控制器")
@ApiSupport(author = "SNOWY_TEAM", order = 1)
@RestController
@Validated
public class DepotRepayBookController {

    @Resource
    private DepotRepayOrderService depotRepayBookService;

    /**
     * 获取还款提货预约分页
     *
     * @author xiang
     * @date  2023/09/14 13:53
     */
    @ApiOperationSupport(order = 1)
    @ApiOperation("获取还款提货预约分页")
    @SaCheckPermission("/biz/depotRepayBook/page")
    @GetMapping("/biz/depotRepayBook/page")
    public CommonResult<Page<DepotRepayOrder>> page(DepotRepayOrderPageParam depotRepayBookPageParam) {
        return CommonResult.data(depotRepayBookService.page(depotRepayBookPageParam));
    }

    /**
     * 添加还款提货预约
     *
     * @author xiang
     * @date  2023/09/14 13:53
     */
    @ApiOperationSupport(order = 2)
    @ApiOperation("添加还款提货预约")
    @CommonLog("添加还款提货预约")
    @SaCheckPermission("/biz/depotRepayBook/add")
    @PostMapping("/biz/depotRepayBook/add")
    public CommonResult<String> add(@RequestBody @Valid DepotRepayOrderAddParam depotRepayBookAddParam) {
        depotRepayBookService.add(depotRepayBookAddParam);
        return CommonResult.ok();
    }

    /**
     * 编辑还款提货预约
     *
     * @author xiang
     * @date  2023/09/14 13:53
     */
    @ApiOperationSupport(order = 3)
    @ApiOperation("编辑还款提货预约")
    @CommonLog("编辑还款提货预约")
    @SaCheckPermission("/biz/depotRepayBook/edit")
    @PostMapping("/biz/depotRepayBook/edit")
    public CommonResult<String> edit(@RequestBody @Valid DepotRepayOrderEditParam depotRepayBookEditParam) {
        depotRepayBookService.edit(depotRepayBookEditParam);
        return CommonResult.ok();
    }

    /**
     * 删除还款提货预约
     *
     * @author xiang
     * @date  2023/09/14 13:53
     */
    @ApiOperationSupport(order = 4)
    @ApiOperation("删除还款提货预约")
    @CommonLog("删除还款提货预约")
    @SaCheckPermission("/biz/depotRepayBook/delete")
    @PostMapping("/biz/depotRepayBook/delete")
    public CommonResult<String> delete(@RequestBody @Valid @NotEmpty(message = "集合不能为空")
                                                   CommonValidList<DepotRepayOrderIdParam> depotRepayBookIdParamList) {
        depotRepayBookService.delete(depotRepayBookIdParamList);
        return CommonResult.ok();
    }

    /**
     * 获取还款提货预约详情
     *
     * @author xiang
     * @date  2023/09/14 13:53
     */
    @ApiOperationSupport(order = 5)
    @ApiOperation("获取还款提货预约详情")
    @SaCheckPermission("/biz/depotRepayBook/detail")
    @GetMapping("/biz/depotRepayBook/detail")
    public CommonResult<DepotRepayOrder> detail(@Valid DepotRepayOrderIdParam depotRepayBookIdParam) {
        return CommonResult.data(depotRepayBookService.detail(depotRepayBookIdParam));
    }

}
