/*
 * Copyright [2022] [https://www.xiaonuo.vip]
 *
 * Snowy采用APACHE LICENSE 2.0开源协议，您在使用过程中，需要注意以下几点：
 *
 * 1.请不要删除和修改根目录下的LICENSE文件。
 * 2.请不要删除和修改Snowy源码头部的版权声明。
 * 3.本项目代码可免费商业使用，商业使用请保留源码和相关描述文件的项目出处，作者声明等。
 * 4.分发源码时候，请注明软件出处 https://www.xiaonuo.vip
 * 5.不可二次分发开源参与同类竞品，如有想法可联系团队xiaonuobase@qq.com商议合作。
 * 6.若您的项目无法满足以上几点，需要更多功能代码，获取Snowy商业授权许可，请在官网购买授权，地址为 https://www.xiaonuo.vip
 */
package vip.xiaonuo.biz.modular.depotSampling.param;

import io.swagger.annotations.ApiModelProperty;
import lombok.Getter;
import lombok.Setter;
import vip.xiaonuo.biz.modular.depotTruckBook.entity.DepotTruckBook;

import java.math.BigDecimal;

/**
 * 质价确认添加参数
 *
 * @author xiang
 * @date  2023/09/13 16:37
 **/
@Getter
@Setter
public class DepotSamplingAddParam {

    /** 项目编码 */
    @ApiModelProperty(value = "项目编码", position = 2)
    private String depotId;

    /** 车辆预约ID */
    @ApiModelProperty(value = "车辆预约ID", position = 3)
    private String truckBookId;

    /** 出入库:入库/出库 */
    @ApiModelProperty(value = "出入库:入库/出库", position = 3)
    private String depotBizType;

    /** 车牌 */
    @ApiModelProperty(value = "车牌", position = 4)
    private String licensePlate;

    /** 司机姓名 */
    @ApiModelProperty(value = "司机姓名", position = 4)
    private String driverName;

    /** 水份 */
    @ApiModelProperty(value = "水份", position = 4)
    private BigDecimal samplingShuiFen;

    /** 容重 */
    @ApiModelProperty(value = "容重", position = 5)
    private String samplingRongZhong;

    /** 不完善率 */
    @ApiModelProperty(value = "不完善率", position = 6)
    private String samplingQueShao;

    /** 霉变 */
    @ApiModelProperty(value = "霉变", position = 7)
    private String samplingMeiBian;

    /** 杂质 */
    @ApiModelProperty(value = "杂质", position = 7)
    private String samplingZaZhi;

    /** 价格 */
    @ApiModelProperty(value = "价格", position = 7)
    private BigDecimal samplingPrice;

    /** 粮食种类 */
    @ApiModelProperty(value = "粮食种类", position = 7)
    private String grainGroup;

    /** 粮食类型 */
    @ApiModelProperty(value = "粮食类型", position = 7)
    private String grainType;

    /** 扦样状态:待扦样/完成扦样/扦样驳回 */
    @ApiModelProperty(value = "扦样状态:待扦样/完成扦样/扦样驳回", position = 9)
    private String samplingType;

    /** 车辆图片进来一 */
    @ApiModelProperty(value = "车辆图片进来一", position = 10)
    private String truckPicInOne;

    /** 车辆图片进来二 */
    @ApiModelProperty(value = "车辆图片进来二", position = 11)
    private String truckPicInTwo;

    /** 车辆图片离开一 */
    @ApiModelProperty(value = "车辆图片离开一", position = 10)
    private String truckPicOutOne;

    /** 车辆图片离开二 */
    @ApiModelProperty(value = "车辆图片离开二", position = 11)
    private String truckPicOutTwo;

    /** 车辆进来时间 */
    @ApiModelProperty(value = "车辆进来时间", position = 14)
    private String truckInTime;

    /** 车辆离开时间 */
    @ApiModelProperty(value = "车辆离开时间", position = 15)
    private String truckOutTime;

    /** 创建人 */
    @ApiModelProperty(value = "创建人", position = 1001)
    private String createUser;

    public DepotSamplingAddParam() {
    }

    public DepotSamplingAddParam(String truckPicInOne, String truckPicInTwo, String truckInTime) {
        this.truckPicInOne = truckPicInOne;
        this.truckPicInTwo = truckPicInTwo;
        this.truckInTime = truckInTime;
    }

    public DepotSamplingAddParam(String truckBookId, String truckPicInOne, String truckPicInTwo, String truckPicOutOne, String truckPicOutTwo, String truckInTime, String truckOutTime) {
        this.truckBookId = truckBookId;
        this.truckPicInOne = truckPicInOne;
        this.truckPicInTwo = truckPicInTwo;
        this.truckPicOutOne = truckPicOutOne;
        this.truckPicOutTwo = truckPicOutTwo;
        this.truckInTime = truckInTime;
        this.truckOutTime = truckOutTime;
    }

    public DepotSamplingAddParam(String truckBookId, BigDecimal samplingShuiFen, String samplingRongZhong, String samplingQueShao, String samplingMeiBian, String samplingZaZhi, BigDecimal samplingPrice) {
        this.truckBookId = truckBookId;
        this.samplingShuiFen = samplingShuiFen;
        this.samplingRongZhong = samplingRongZhong;
        this.samplingQueShao = samplingQueShao;
        this.samplingMeiBian = samplingMeiBian;
        this.samplingZaZhi = samplingZaZhi;
        this.samplingPrice = samplingPrice;
    }

    public DepotSamplingAddParam asByBook(DepotTruckBook book) {
        this.depotId = book.getDepotId();
        this.truckBookId = book.getTruckBookId();
        this.depotBizType = book.getDepotBizType();
        this.licensePlate = book.getLicensePlate();
        this.driverName = book.getDriverName();
        this.grainGroup = book.getGrainGroup();
        this.grainType = book.getGrainType();
        return this;
    }
}
