/*
 * Copyright [2022] [https://www.xiaonuo.vip]
 *
 * Snowy采用APACHE LICENSE 2.0开源协议，您在使用过程中，需要注意以下几点：
 *
 * 1.请不要删除和修改根目录下的LICENSE文件。
 * 2.请不要删除和修改Snowy源码头部的版权声明。
 * 3.本项目代码可免费商业使用，商业使用请保留源码和相关描述文件的项目出处，作者声明等。
 * 4.分发源码时候，请注明软件出处 https://www.xiaonuo.vip
 * 5.不可二次分发开源参与同类竞品，如有想法可联系团队xiaonuobase@qq.com商议合作。
 * 6.若您的项目无法满足以上几点，需要更多功能代码，获取Snowy商业授权许可，请在官网购买授权，地址为 https://www.xiaonuo.vip
 */
package vip.xiaonuo.biz.modular.depotTruckLade.controller;

import cn.dev33.satoken.annotation.SaCheckPermission;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.github.xiaoymin.knife4j.annotations.ApiOperationSupport;
import com.github.xiaoymin.knife4j.annotations.ApiSupport;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RestController;
import vip.xiaonuo.biz.modular.depotTruckLade.param.*;
import vip.xiaonuo.common.annotation.CommonLog;
import vip.xiaonuo.common.pojo.CommonResult;
import vip.xiaonuo.common.pojo.CommonValidList;
import vip.xiaonuo.biz.modular.depotTruckLade.entity.DepotTruckLade;
import vip.xiaonuo.biz.modular.depotTruckLade.service.DepotTruckLadeService;

import javax.annotation.Resource;
import javax.validation.Valid;
import javax.validation.constraints.NotEmpty;

/**
 * 车辆装卸货管理表控制器
 *
 * @author xiang
 * @date  2023/10/16 14:18
 */
@Api(tags = "车辆装卸货管理表控制器")
@ApiSupport(author = "SNOWY_TEAM", order = 1)
@RestController
@Validated
public class DepotTruckLadeController {

    @Resource
    private DepotTruckLadeService depotTruckLadeService;

    /**
     * 获取车辆装卸货管理表分页
     *
     * @author xiang
     * @date  2023/10/16 14:18
     */
    @ApiOperationSupport(order = 1)
    @ApiOperation("获取车辆装卸货管理表分页")
    @SaCheckPermission("/biz/depotTruckLade/page")
    @GetMapping("/biz/depotTruckLade/page")
    public CommonResult<Page<DepotTruckLade>> page(DepotTruckLadePageParam depotTruckLadePageParam) {
        return CommonResult.data(depotTruckLadeService.page(depotTruckLadePageParam));
    }

    /**
     * 添加车辆装卸货管理表
     *
     * @author xiang
     * @date  2023/10/16 14:18
     */
    @ApiOperationSupport(order = 2)
    @ApiOperation("添加车辆装卸货管理表")
    @CommonLog("添加车辆装卸货管理表")
    @SaCheckPermission("/biz/depotTruckLade/add")
    @PostMapping("/biz/depotTruckLade/add")
    public CommonResult<String> add(@RequestBody @Valid DepotTruckLadeAddParam depotTruckLadeAddParam) {
        depotTruckLadeService.add(depotTruckLadeAddParam);
        return CommonResult.ok();
    }

    /**
     * 编辑车辆装卸货管理表
     *
     * @author xiang
     * @date  2023/10/16 14:18
     */
    @ApiOperationSupport(order = 3)
    @ApiOperation("编辑车辆装卸货管理表")
    @CommonLog("编辑车辆装卸货管理表")
    @SaCheckPermission("/biz/depotTruckLade/edit")
    @PostMapping("/biz/depotTruckLade/edit")
    public CommonResult<String> edit(@RequestBody @Valid DepotTruckLadeEditParam depotTruckLadeEditParam) {
        depotTruckLadeService.edit(depotTruckLadeEditParam);
        return CommonResult.ok();
    }

    /**
     * 删除车辆装卸货管理表
     *
     * @author xiang
     * @date  2023/10/16 14:18
     */
    @ApiOperationSupport(order = 4)
    @ApiOperation("删除车辆装卸货管理表")
    @CommonLog("删除车辆装卸货管理表")
    @SaCheckPermission("/biz/depotTruckLade/delete")
    @PostMapping("/biz/depotTruckLade/delete")
    public CommonResult<String> delete(@RequestBody @Valid @NotEmpty(message = "集合不能为空")
                                                   CommonValidList<DepotTruckLadeIdParam> depotTruckLadeIdParamList) {
        depotTruckLadeService.delete(depotTruckLadeIdParamList);
        return CommonResult.ok();
    }

    /**
     * 获取车辆装卸货管理表详情
     *
     * @author xiang
     * @date  2023/10/16 14:18
     */
    @ApiOperationSupport(order = 5)
    @ApiOperation("获取车辆装卸货管理表详情")
    @SaCheckPermission("/biz/depotTruckLade/detail")
    @GetMapping("/biz/depotTruckLade/detail")
    public CommonResult<DepotTruckLade> detail(@Valid DepotTruckLadeIdParam depotTruckLadeIdParam) {
        return CommonResult.data(depotTruckLadeService.detail(depotTruckLadeIdParam));
    }
    /**
     * 添加车辆装卸货一次照片
     *
     * @author xiang
     * @date 2023/10/16 11:40
     */
    @ApiOperationSupport(order = 2)
    @ApiOperation("添加车辆装卸货一次照片")
    @CommonLog("添加车辆装卸货一次照片")
    @SaCheckPermission("/biz/depotTruckLade/add")
    @PostMapping("/biz/depotTruckLade/addOnePic")
    public CommonResult<String> addOnePic(@RequestBody @Valid DepotTruckLadeAddPicParam param) {
        depotTruckLadeService.addOnePic(param);
        return CommonResult.ok();
    }

    /**
     * 添加车辆装卸货二次照片
     *
     * @author xiang
     * @date 2023/10/16 11:40
     */
    @ApiOperationSupport(order = 2)
    @ApiOperation("添加车辆装卸货二次照片")
    @CommonLog("添加车辆装卸货二次照片")
    @SaCheckPermission("/biz/depotTruckLade/add")
    @PostMapping("/biz/depotTruckLade/addTwoPic")
    public CommonResult<String> addTwoPic(@RequestBody @Valid DepotTruckLadeAddPicParam param) {
        param.setShedName(null);
        depotTruckLadeService.addTwoPic(param);
        return CommonResult.ok();
    }

}
