/*
 * Copyright [2022] [https://www.xiaonuo.vip]
 *
 * Snowy采用APACHE LICENSE 2.0开源协议，您在使用过程中，需要注意以下几点：
 *
 * 1.请不要删除和修改根目录下的LICENSE文件。
 * 2.请不要删除和修改Snowy源码头部的版权声明。
 * 3.本项目代码可免费商业使用，商业使用请保留源码和相关描述文件的项目出处，作者声明等。
 * 4.分发源码时候，请注明软件出处 https://www.xiaonuo.vip
 * 5.不可二次分发开源参与同类竞品，如有想法可联系团队xiaonuobase@qq.com商议合作。
 * 6.若您的项目无法满足以上几点，需要更多功能代码，获取Snowy商业授权许可，请在官网购买授权，地址为 https://www.xiaonuo.vip
 */
package vip.xiaonuo.biz.modular.depotTruckWeighPlug.controller;

import cn.dev33.satoken.annotation.SaCheckPermission;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.github.xiaoymin.knife4j.annotations.ApiOperationSupport;
import com.github.xiaoymin.knife4j.annotations.ApiSupport;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RestController;
import vip.xiaonuo.biz.modular.depotTruckWeighPlug.param.*;
import vip.xiaonuo.common.annotation.CommonLog;
import vip.xiaonuo.common.pojo.CommonResult;
import vip.xiaonuo.common.pojo.CommonValidList;
import vip.xiaonuo.biz.modular.depotTruckWeighPlug.entity.DepotTruckWeighPlug;
import vip.xiaonuo.biz.modular.depotTruckWeighPlug.service.DepotTruckWeighPlugService;

import javax.annotation.Resource;
import javax.validation.Valid;
import javax.validation.constraints.NotEmpty;

/**
 * 车辆称重补单表控制器
 *
 * @author xiang
 * @date 2023/11/27 08:54
 */
@Api(tags = "车辆称重补单表控制器")
@ApiSupport(author = "SNOWY_TEAM", order = 1)
@RestController
@Validated
public class DepotTruckWeighPlugController {

    @Resource
    private DepotTruckWeighPlugService depotTruckWeighPlugService;

    /**
     * 获取车辆称重补单表分页
     *
     * @author xiang
     * @date 2023/11/27 08:54
     */
    @ApiOperationSupport(order = 1)
    @ApiOperation("获取车辆称重补单表分页")
    @SaCheckPermission("/biz/depotTruckWeighPlug/page")
    @GetMapping("/biz/depotTruckWeighPlug/page")
    public CommonResult<Page<DepotTruckWeighPlug>> page(DepotTruckWeighPlugPageParam depotTruckWeighPlugPageParam) {
        return CommonResult.data(depotTruckWeighPlugService.page(depotTruckWeighPlugPageParam));
    }

    /**
     * 添加车辆称重补单表
     *
     * @author xiang
     * @date 2023/11/27 08:54
     */
    @ApiOperationSupport(order = 2)
    @ApiOperation("添加车辆称重补单表")
    @CommonLog("添加车辆称重补单表")
    @SaCheckPermission("/biz/depotTruckWeighPlug/add")
    @PostMapping("/biz/depotTruckWeighPlug/add")
    public CommonResult<String> add(@RequestBody @Valid DepotTruckWeighPlugAddParam depotTruckWeighPlugAddParam) {
        depotTruckWeighPlugService.add(depotTruckWeighPlugAddParam);
        return CommonResult.ok();
    }

    /**
     * 编辑车辆称重补单表
     *
     * @author xiang
     * @date 2023/11/27 08:54
     */
    @ApiOperationSupport(order = 3)
    @ApiOperation("编辑车辆称重补单表")
    @CommonLog("编辑车辆称重补单表")
    @SaCheckPermission("/biz/depotTruckWeighPlug/edit")
    @PostMapping("/biz/depotTruckWeighPlug/edit")
    public CommonResult<String> edit(@RequestBody @Valid DepotTruckWeighPlugEditParam depotTruckWeighPlugEditParam) {
        depotTruckWeighPlugService.edit(depotTruckWeighPlugEditParam);
        return CommonResult.ok();
    }

    /**
     * 删除车辆称重补单表
     *
     * @author xiang
     * @date 2023/11/27 08:54
     */
    @ApiOperationSupport(order = 4)
    @ApiOperation("删除车辆称重补单表")
    @CommonLog("删除车辆称重补单表")
    @SaCheckPermission("/biz/depotTruckWeighPlug/delete")
    @PostMapping("/biz/depotTruckWeighPlug/delete")
    public CommonResult<String> delete(@RequestBody @Valid @NotEmpty(message = "集合不能为空")
                                               CommonValidList<DepotTruckWeighPlugIdParam> depotTruckWeighPlugIdParamList) {
        depotTruckWeighPlugService.delete(depotTruckWeighPlugIdParamList);
        return CommonResult.ok();
    }

    /**
     * 获取车辆称重补单表详情
     *
     * @author xiang
     * @date 2023/11/27 08:54
     */
    @ApiOperationSupport(order = 5)
    @ApiOperation("获取车辆称重补单表详情")
    @SaCheckPermission("/biz/depotTruckWeighPlug/detail")
    @GetMapping("/biz/depotTruckWeighPlug/detail")
    public CommonResult<DepotTruckWeighPlug> detail(@Valid DepotTruckWeighPlugIdParam depotTruckWeighPlugIdParam) {
        return CommonResult.data(depotTruckWeighPlugService.detail(depotTruckWeighPlugIdParam));
    }

    /**
     * 车辆称重补单审批
     *
     * @author xiang
     * @date 2023/11/27 08:54
     */
    @ApiOperationSupport(order = 5)
    @ApiOperation("车辆称重补单审批")
    @CommonLog("车辆称重补单审批")
    @SaCheckPermission("/biz/depotTruckWeighPlug/edit")
    @PostMapping("/biz/depotTruckWeighPlug/bankOpApproval")
    public CommonResult<String> bankOpApproval(@RequestBody @Valid DepotTruckWeighPlugEditParam param) {
        depotTruckWeighPlugService.bankOpApproval(param);
        return CommonResult.ok();
    }
}
